/****************************************************************************
**	Includedatei zu DateValidator										*
**	in der Datei :  DateValditor.cpp										*
**																			*
**	(c) Arne Stocker														*
**																			*
*****************************************************************************/

#ifndef DATE_VALIDATOR_H
#define DATE_VALIDATOR_H

#include <QValidator>
#include <QLineEdit>
#include <QDateTime>
#include <QKeyEvent>

const QDate MIN_DATE = QDate(1800,1,1);
const QDate MAX_DATE = QDate(4000,1,1);

QString DateToText(QDate date);
QDate TextToDate(QString text,QChar trennzeichen);

class DateValidator: public QValidator
{
    friend class Kalender;

    Q_OBJECT

	// Konstruktor und Destruktor

	public:
        DateValidator( QWidget * parent );
		~DateValidator();

	// statische Umrechenmethoden

	public:
		static bool		IsValid(const QString &text);

    signals:

        void dateChanged(const QDate&);
		
	// Schnittstelle

	public:
		
        void SetRange(QDate minDate, QDate maxDate);
		void SetLineEdit(QLineEdit *pEditLine);
        void SetJahrhunderGrenze(int grenze);
        void SetTrennzeichen(char trennzeichen); 
        void SetKeinJahrEnabled(bool enable);
        void SetActualDate(QDate actualDate);

	// virtuelle Methoden zum Prüfen und korrigieren
	
	protected:

        virtual QValidator::State validate( QString &, int & ) const;
        virtual void fixup ( QString & input ) const ;

	// Ereignisfilter für das überwachte Editfeld

	protected:
		virtual bool eventFilter ( QObject *, QEvent * ) ;

    protected : 

        void SetValidate(bool on);

	private :
		QDate m_minDate;
		QDate m_maxDate;
        QDate m_actuallDate;
        
        QChar m_trennzeichen;
        int   m_jahrhundertGrenze;
        bool  m_keinJahrEnabled;
        bool  m_eventFilterOn;
        bool  m_lastRelease;

        static bool m_noValidate;
};


#endif		// DATE_VALIDATOR_H
